
const subTableId = "#subTable";
const subModalId = "#subFormModal";


// region Init Module

function initSubscriptionsModule() {
    initializeDataTable(
        subTableId,
        [
            {
                title: "Name",
                data: "name",
                className: "text-truncate col-3",
                render: function(data, type, row) {
                    const name = sanitise(data);
                    return `<button type="button" class="btn btn-link text-start text-decoration-none edit-modal">${name}</button>`;
                }
            },
            {
                title: "URL",
                data: "url",
                className: "text-truncate col-4",
                render: function(data, type) {
                    const url = sanitise(data);
                    return `<a href="${url}" class="btn btn-link text-start text-decoration-none" target="_blank">${url}</a>`;
                }
            },
            {
                title: "Rules",
                data: "unique_rules",
                className: "text-center text-nowrap col-1",
                render: function(data, type) {
                    let badges = $("<div>");
                    data.forEach(function(rule, idx) {
                        let badge = $(`<span class="badge text-bg-secondary">${rule}</span>`)  // TODO: translate to rule.name
                        if (idx > 0) { badge.addClass("ms-2") }
                        badges.append(badge);
                    });

                    return badges.html();
                }
            },
            {
                title: "Created At",
                data: "created_at",
                className: "col-3",
                render: function(data, type) {
                    let dateTime = new Date(data);
                    return $(`
                        <span data-bs-trigger="hover focus"
                            data-bs-html="true"
                            data-bs-custom-class="text-center"
                            data-bs-toggle="popover"
                            data-bs-content="${formatStringDate(dateTime, "%a, %D %B, %Y<br>%H:%M:%S")}">
                            ${formatStringDate(dateTime, "%D, %b %Y")}
                        </span>
                    `).popover()[0];
                }
            },
            { 
                title: "Enabled",
                data: "active",
                orderable: false,
                className: "text-center form-switch col-1",
                render: function(data, type) {
                    return `<input type="checkbox" class="sub-toggle-active form-check-input ms-0" ${data ? "checked" : ""} />`
                }
            }
        ]
    );
}


// region Load Data

$(document).on("selectedServerChange", async function() {
    await loadSubscriptionData();
});

$(subTableId).on("doDataLoad", async function() {
    await loadSubscriptionData();
})

async function loadSubscriptionData() {
    if (!selectedServer) {
        return;
    }

    setTableFilter(subTableId, "server", selectedServer.id);
    await loadTableData(subTableId, `/api/subscriptions/`, "GET");
}


// region Table Switches

$(subTableId).on("change", ".sub-toggle-active", async function() {
    let active = $(this).prop("checked");
    let id = $(subTableId).DataTable().row($(this).closest("tr")).data().id;
    let sub = await ajaxRequest(`/api/subscriptions/${id}/`, "GET");
    sub.active = active;
    let formData = objectToFormData(sub);
    await ajaxRequest(`/api/subscriptions/${id}/`, "PUT", formData);
});


// region New/Edit Modal

$(subTableId).closest('.js-tableBody').siblings('.js-tableFilters').on("click", ".table-new-btn", async function() {
    await openDataModal(subModalId, -1);
});

$(subTableId).on("click", ".edit-modal", async function() {
    let id = $(subTableId).DataTable().row($(this).closest("tr")).data().id;
    await openDataModal(subModalId, id, `/api/subscriptions/${id}/`);
})

async function openSubModal(id) {
    $modal = $(subModalId);
    $modal.find('input[data-role="is-id"]').val(id);

    if (parseInt(id) === -1) {
        $modal.find(".form-create").show();
        $modal.find(".form-edit").hide();

        $modal.find("[data-field]").each(function() {
            const type = $(this).attr("type");
            const defaultVal = $(this).attr("data-default") || "";

            if (type === "checkbox") {
                $(this).prop("checked", defaultVal === "true");
            }
            else if (type === "datetime-local") {
                $(this).val(getCurrentDateTime());
            }
            else {
                $(this).val(defaultVal).change();
            }
        });
    }
    else {
        $modal.find(".form-create").hide();
        $modal.find(".form-edit").show();

        const data = await ajaxRequest(`/api/subscriptions/${id}/`, "GET");

        $modal.find("[data-field]").each(function() {
            const key = $(this).attr("data-field");
            const value = data[key];

            if (typeof value === "boolean") {
                $(this).prop("checked", value);
            }
            else if (isISODateTimeString(value)) {
                console.log(value.split('+')[0].substring(0, 16));
                $(this).val(value.split('+')[0].substring(0, 16));
            }
            else {
                $(this).val(value).change();
            }
        });
    }

    $modal.modal("show");
}

$(subModalId).on("submit", async function(event) {
    event.preventDefault();

    if (!selectedServer) {
        throw new Error("Cannot submit form while no server is selected");
    }

    let data = { server: selectedServer.id };

    $(this).find("[data-field]").each(function() {
        const type = $(this).attr("type");
        const key = $(this).attr("data-field");
        if (!key) {
            return;
        }

        let value;
        if (type === "checkbox") {
            value = $(this).prop("checked");
        }
        else {
            value = $(this).val();
        }

        data[key] = value;
    });

    const formData = objectToFormData(data);
    const id = $(this).find('input[data-role="is-id"]').val();

    if (parseInt(id) !== -1) {
        ajaxRequest(`/api/subscriptions/${id}/`, "PATCH", formData)
            .then(response => {
                $(this).modal("hide");
                $(subTableId).trigger("doDataLoad");
            })
            .catch(error => logError(error))
    }
    else {
        ajaxRequest("/api/subscriptions/", "POST", formData)
            .then(response => {
                $(this).modal("hide");
                $(subTableId).trigger("doDataLoad");
            })
            .catch(error => logError(error));
    }
});


// region Load Modal Options

$(document).on("selectedServerChange", async function() {
    await loadMessageStyleOptions();
});

async function loadMessageStyleOptions() {
    let $input = $(subModalId).find('[data-field="message_style"]');

    // Disable and clear input
    $input.val("").change();
    $input.prop("disabled", true);

    // Delete existing options
    $input.find("option").each(function() {
        if ($(this).val()) {
            $(this).remove();
        }
    });

    // Load new values
    const data = await ajaxRequest(`/api/message-styles/?server=${selectedServer.id}`, "GET");
    data.results.forEach(style => {
        $input.append($(
            "<option>",
            {text: style.name, value: style.id}
        ));
    });

    // Re-enable input
    $input.prop("disabled", false);
}

$(document).ready(async function() {
    await loadUniqueRules();
})

async function loadUniqueRules() {
    let $input = $(subModalId).find('[data-field="unique_rules"]');
    $input.prop("disabled", true);

    // Load new values
    const data = await ajaxRequest("/api/unique-content-rules/", "GET");
    data.results.forEach(style => {
        $input.append($(
            "<option>",
            {text: style.name, value: style.id}
        ));
    });

    $input.prop("disabled", false);
}
